<?php
/*

Threshold plugin 2.1a for cacti 8.6c

  Library functions for THold
  
*/

// Update automatically 'alert_base_url' if not set and if we are called from the browser
// so that check-thold can pick it up
if(isset($_SERVER["HTTP_HOST"]) && isset($_SERVER["PHP_SELF"]) && read_config_option("alert_base_url") == "") {
	db_execute("replace into settings (name,value) values ('alert_base_url', '" . ("http://" . $_SERVER["HTTP_HOST"] . dirname($_SERVER["PHP_SELF"]) . "/") . "')");
	
	/* reset local settings cache so the user sees the new settings */
	kill_session_var("sess_config_array");
}

function rrd_last($rra, $cf)
{
	global $config;
	$last_time_entry = exec(read_config_option("path_rrdtool") . " last " . get_data_source_path($rra, true) . " " . $cf);
	
	return $last_time_entry;
}

function get_current_value($rra, $ds)
{
	global $config;
	$last_time_entry = rrd_last($rra, "MAX");
	$last_needed = $last_time_entry + 900;
	
	$result = rrdtool_function_fetch($rra, time() - 900, time());
	// Return 0 if the data source is not found (Newly created?) 
	if (!isset( $result["data_source_names"])) return false; 
//	var_dump($result); die();
	$idx = array_search($ds, $result["data_source_names"]); 

	// Return 0 if the value was not found (Cache Cleared?) 
	if (!isset($result["values"][$idx][0])) return false; 
    
	return round($result["values"][$idx][0]);
}

function thold_get_ref_value($rra_id, $ds, $ref_time, $time_range)
{
	global $config;
	
	$real_ref_time = time() - $ref_time;
	
	$result = rrdtool_function_fetch($rra_id, $real_ref_time - ($time_range / 2), $real_ref_time + ($time_range / 2));
	
	//print_r($result);
	//echo "\n";
	$idx = array_search($ds, $result["data_source_names"]);
	if(count($result["values"][$idx]) == 0) {
		return false;
	}
	
	return $result["values"][$idx];
}

/* thold_check_exception_periods
 @to-do: This function should check "globally" declared exceptions, like
 holidays etc., as well as exceptions bound to the speciffic $rra_id. $rra_id 
 should inherit exceptions that are assigned on the higher level (i.e. device).
 
*/
function thold_check_exception_periods($rra_id, $ref_time, $ref_range)
{
	// TO-DO
	// Check if the reference time falls into global exceptions
	// Check if the current time falls into global exceptions
	// Check if $rra_id + $ds have an exception (again both reference time and current time)
	// Check if there are inheritances
	
	// More on the exception concept:
	// -Exceptions can be one time and recurring
	// -Exceptions can be global and assigned to:
	// 	-templates
	//	-devices
	//	-data sources
	//

	return false;
}

/* thold_check_baseline - 
 Should be called after hard limits have been checked and only when they are OK
 
 The function "goes back in time" $ref_time seconds and retrieves the data 
 for $ref_range seconds. Then it finds minimum and maximum values and calculates
 allowed deviations from those values.
 
 @arg $rra_id - the data source to check the data
 @arg $ds - Index of the data_source in the RRD
 @arg $ref_time - Integer value representing reference offset in seconds
 @arg $ref_range - Integer value indicating reference time range in seconds
 @arg $current_value - Current "value" of the data source
 @arg $pct_down - Allowed baseline deviation in % - if set to false will not be considered
 @arg $pct_up - Allowed baseline deviation in % - if set to false will not be considered
 
 @returns (integer) - integer value that indicates status
   -2 if the exception is active
   -1 if the reference value is not available
   0 if the current value is within the boundaries
   1 if the current value is below the calculated threshold
   2 if the current value is above the calculated threshold
 */
function &thold_check_baseline($rra_id, $ds, $ref_time, $ref_range, $current_value, $pct_down, $pct_up)
{
	global $debug;
	
	// First let's check if either current time or reference time falls within either
	// globally set exceptions or rra itself has some exceptios
	
	if(thold_check_exception_periods($rra_id, $ref_time, $ref_range)) {
		return -2;	// An exception period is blocking us out...
	}
	$ref_values = thold_get_ref_value($rra_id, $ds, $ref_time, $ref_range);
	
	if(!$ref_values) {
		// if($debug) echo "Baseline reference value not yet established!\n";
		return -1; // Baseline reference value not yet established
	}
	
	$ref_value_max = round(max($ref_values));
	$ref_value_min = round(min($ref_values));
	
	$blt_low = false;
	$blt_high = false;
	
	if($pct_down != "") {
		$blt_low = round($ref_value_min - ($ref_value_min * $pct_down / 100));
	}
	
	if($pct_up != "") {
		$blt_high = round($ref_value_max + ($ref_value_max * $pct_up / 100));
	}
	
	$failed = 0;
	
	// Check low boundary
	if($blt_low && $current_value < $blt_low) {
		$failed = 1;
	}
	
	// Check up boundary
	if($failed == 0 && $blt_high && $current_value > $blt_high) {
		$failed = 2;
	}
	
	if($debug) {
		echo "RRA: $rra_id : $ds\n";
		echo "Ref. values count: ". count($ref_values) . "\n";
		echo "Ref. value (min): $ref_value_min\n";
		echo "Ref. value (max): $ref_value_max\n";
		echo "Cur. value: $current_value\n";
		echo "Low bl thresh: $blt_low\n";
		echo "High bl thresh: $blt_high\n";
		echo "Check against baseline: ";
		switch($failed) {
			case 0:
			echo "OK";
			break;
			
			case 1:
			echo "FAIL: Below baseline threshold!";
			break;
			
			case 2:
			echo "FAIL: Above baseline threshold!";
			break;
		}
		echo "\n";
		echo "------------------\n";
	}
	
	return $failed;
}

function save_thold()
{
        global $rra;
        global $banner;
	global $hostid;

        // Make sure this is defined
        $_POST["bl_enabled"] = isset($_POST["bl_enabled"]) ? $_POST["bl_enabled"] : "off";

        // --------------------------------------------------------------------------------
        // Perform checks (to prevent having people wake up at night due to wrong settings)
        // --------------------------------------------------------------------------------
        $banner = "<font color=red><strong>";
        if((!isset($_POST['thold_hi']) || trim($_POST['thold_hi']) == "") &&
                (!isset($_POST['thold_low']) || trim($_POST['thold_low']) == "") &&
			(!isset($_POST['bl_ref_time']) || trim($_POST['bl_ref_time'])  == "")) {
                $banner .= "You must specify either &quot;High Threshold&quot; or &quot;Low Threshold&quot; or both!<br>RECORD NOT UPDATED!</strong></font>";
                return;
        }

        if(isset($_POST['thold_hi']) && isset($_POST['thold_low']) &&
                trim($_POST['thold_hi']) != "" && trim($_POST['thold_low']) != "" &&
                $_POST['thold_low'] >= $_POST['thold_hi']) {
                $banner .= "Impossible thresholds: &quot;High Threshold&quot; smaller than or equal to &quot;Low Threshold&quot;<br>RECORD NOT UPDATED!</strong></font>";
                return;
        }

        if($_POST["bl_enabled"] == "on") {
                $banner .= "With baseline thresholds enabled ";

                if(!thold_mandatory_field_ok("bl_ref_time", "Reference in the past")) {
                        return;
                }

                if((!isset($_POST['bl_pct_down']) || trim($_POST['bl_pct_down']) == "") &&
                        (!isset($_POST['bl_pct_up']) || trim($_POST['bl_pct_up']) == "")) {
                        $banner .= "You must specify either &quot;Baseline deviation UP&quot; or &quot;Baseline deviation DWON&quot; or both!<br>RECORD NOT UPDATED!</strong></font>";
                        return;
                }

        }

        $existing = db_fetch_assoc("SELECT id FROM thold WHERE rra = " . $rra);
        if (count($existing) == 0) {
                $sql = "INSERT INTO thold (element, rra) VALUES ('" . $_POST["element"] . "', '" . $rra . "')";

        } else {
                $sql = "UPDATE thold SET element='" . $_POST["element"] . "' WHERE rra=$rra";

        };
        mysql_query($sql) or die("Update failed: <pre>" . $sql . "\n" . mysql_error() . "</pre>");

        $existing = db_fetch_assoc("SELECT id FROM thold_data WHERE rra_id = " . $rra . " AND data_id = " . $_POST["data_template_rrd_id"]);
        if (count($existing) == 0) {
                $sql = "INSERT INTO thold_data (host_id, rra_id, data_id, thold_hi, thold_low, thold_fail_trigger, bl_enabled, repeat_alert, notify_default, notify_extra";
                if($_POST["bl_enabled"] == "on") {
                        $sql .= ", bl_ref_time, bl_ref_time_range, bl_pct_down, bl_pct_up, bl_fail_trigger";
                }
                $sql .= ") ";
                $sql .= "VALUES('$hostid', '$rra'";
                $sql .= ", '" . $_POST["data_template_rrd_id"] . "'";
                $sql .= ", " . (trim($_POST["thold_hi"]) == "" ? "NULL" : ("'" . $_POST["thold_hi"] . "'"));
                $sql .= ", " . (trim($_POST["thold_low"]) == "" ? "NULL" : ("'" . $_POST["thold_low"] . "'"));
                $sql .= ", " . (trim($_POST["thold_fail_trigger"]) == "" ? "NULL" : ("'" . $_POST["thold_fail_trigger"] . "'"));
                $sql .= ", '" . $_POST["bl_enabled"] . "'";
                $sql .= ", " . (trim($_POST["repeat_alert"]) == "" ? "NULL" : ("'" . $_POST["repeat_alert"] . "'"));
                $sql .= ", " . ($_POST["notify_default"] == "NULL" ? "NULL" : ("'" . $_POST["notify_default"] . "'"));
                $sql .= ", " . (trim($_POST["notify_extra"]) == "" ? "NULL" : ("'" . $_POST["notify_extra"] . "'"));
                if($_POST["bl_enabled"] == "on") {
                        $sql .= ", '" . $_POST["bl_ref_time"] . "'";
                        $sql .= ", '" . $_POST["bl_ref_time_range"] . "'";
                        $sql .= ", " . (trim($_POST["bl_pct_down"]) == "" ? "NULL" : ("'" . $_POST["bl_pct_down"] . "'"));
                        $sql .= ", " . (trim($_POST["bl_pct_up"]) == "" ? "NULL" : ("'" . $_POST["bl_pct_up"] . "'"));
                        $sql .= ", " . (trim($_POST["bl_fail_trigger"]) == "" ? "NULL" : ("'" . $_POST["bl_fail_trigger"] . "'"));
                }
                $sql .= ")";

        } else {
                $sql = "UPDATE thold_data ";
                $sql .= "SET thold_hi=" . (trim($_POST["thold_hi"]) == "" ? "NULL" : ("'" . $_POST["thold_hi"] . "'"));
                $sql .= ", thold_low=" . (trim($_POST["thold_low"]) == "" ? "NULL" : ("'" . $_POST["thold_low"] . "'"));
                $sql .= ", thold_fail_trigger=" . (trim($_POST["thold_fail_trigger"]) == "" ? "NULL" : ("'" . $_POST["thold_fail_trigger"] . "'"));
                $sql .= ", bl_enabled='" . $_POST["bl_enabled"] . "'";
                $sql .= ", repeat_alert=" . (trim($_POST["repeat_alert"]) == "" ? "NULL" : ("'" . $_POST["repeat_alert"] . "'"));
                $sql .= ", notify_default=" . ($_POST["notify_default"] == "NULL" ? "NULL" : ("'" . $_POST["notify_default"] . "'"));
                $sql .= ", notify_extra=" . (trim($_POST["notify_extra"]) == "" ? "NULL" : ("'" . $_POST["notify_extra"] . "'"));
                if($_POST["bl_enabled"] == "on") {
                        $sql .= ", bl_ref_time='" . $_POST["bl_ref_time"] . "'";
                        $sql .= ", bl_ref_time_range='" . $_POST["bl_ref_time_range"] . "'";
                        $sql .= ", bl_pct_down=" . (trim($_POST["bl_pct_down"]) == "" ? "NULL" : ("'" . $_POST["bl_pct_down"] . "'"));
                        $sql .= ", bl_pct_up=" . (trim($_POST["bl_pct_up"]) == "" ? "NULL" : ("'" . $_POST["bl_pct_up"] . "'"));
                        $sql .= ", bl_fail_trigger=" . (trim($_POST["bl_fail_trigger"]) == "" ? "NULL" : ("'" . $_POST["bl_fail_trigger"] . "'"));
                }
                $sql .= " WHERE rra_id = '" . $rra . "' AND data_id = '" . $_POST["data_template_rrd_id"] . "'";


        }
        mysql_query($sql) or die("Update failed: <pre>" . $sql . "\n" . mysql_error() . "</pre>");

        $banner="<font color=green><strong>Record Updated</strong></font>";
}


function thold_mandatory_field_ok($name, $friendly_name)
{
        global $banner;
        if(!isset($_POST[$name]) ||
                (isset($_POST[$name]) &&
                        (trim($_POST[$name]) == "" ||
                $_POST[$name] <= 0))) {
                $banner .= "&quot;" . $friendly_name . "&quot; must be set to positive integer value!<br>RECORD NOT UPDATED!</strong></font>";
                return false;
        }
        return true;
}

// Create tholds for all possible data elements for a host
function autocreate($hostid) {
$alert_bl_past_default = read_config_option("alert_bl_past_default");
$alert_bl_timerange_def = read_config_option("alert_bl_timerange_def");
$alert_bl_percent_def = read_config_option("alert_bl_percent_def");
$alert_bl_trigger = read_config_option("alert_bl_trigger");
        $rralist=mysql_query("SELECT id FROM data_local where host_id='$hostid'") or die(mysql_error());
        while ($row = mysql_fetch_array($rralist, MYSQL_ASSOC)) {
	$local_data_id=$row["id"];
	$existing = db_fetch_assoc("SELECT id FROM thold WHERE rra = " . $local_data_id);
                if (count($existing) == 0) {
                        $rrdsql = mysql_fetch_array(mysql_query("SELECT id FROM data_template_rrd WHERE local_data_id=$local_data_id LIMIT 1"));
                        $rrdlookup = $rrdsql["id"];
                        $grapharr = mysql_fetch_array(mysql_query("SELECT local_graph_id FROM graph_templates_item WHERE task_item_id=$rrdlookup LIMIT 1"));
                        $graph = $grapharr["local_graph_id"];
			
			$sql = "INSERT INTO thold (element, rra) VALUES ($graph, $local_data_id)";
                        mysql_query($sql)  or die("Update failed: <pre>" . $sql . "\n" . mysql_error() . "</pre>");;

	                $rrdlist=mysql_query("SELECT id FROM data_template_rrd where local_data_id='$local_data_id'") or die(mysql_error());
	                while ($rrdrow =  mysql_fetch_array($rrdlist, MYSQL_ASSOC)) {
                        $data_rrd_id=$rrdrow["id"];
			      $existing = db_fetch_assoc("SELECT id FROM thold_data WHERE rra_id='$local_data_id' AND data_id='$data_rrd_id'");
			      if (count($existing) == 0) {
		                        $sql = "INSERT INTO thold_data (host_id, rra_id, data_id, bl_enabled, bl_ref_time, bl_ref_time_range, bl_pct_down, bl_pct_up, bl_fail_trigger) ";
		                        $sql .= " VALUES ('$hostid', '$local_data_id', '$data_rrd_id', 'on', '$alert_bl_past_default', '$alert_bl_timerange_def', '$alert_bl_percent_def', '$alert_bl_percent_def', '$alert_bl_trigger')";
		                        echo "Created threshold for data_id $local_data_id and rrd_id $data_rrd_id <Br>";
		                        mysql_query($sql) or die("Update failed: <pre>" . $sql . "\n" . mysql_error() . "</pre>");
				}
                        }
                }
        }
}
?>
